import com.bjc.offbeat.client.*;
import java.io.*;

public class SimpleChat extends Thread
{
  private OffbeatClient oc;
	private String name = new String();
  
	public static void main( String[] args )
	{
	 		SimpleChat sc;
			
	 		if( args.length == 3 )
			{
	 		 		sc = new SimpleChat( args[0], Integer.parseInt( args[1] ), args[2] );
			}
			else
			{
			 		System.out.println( "Usage: java SimpleChat <host> <port> <your name>" );
			}
	}
	
  public SimpleChat( String host, int port, String name )
  {
	 	this.name = name;
    this.oc = new OffbeatClient( this, host, port );
    this.oc.onPushMessage = "pushMsgHandler";
		this.oc.onConnect = "connected";
    this.oc.onClose = "closed";
    this.oc.onClientDisconnect = "clientDisconnected";
    this.oc.debug = 0;
		this.oc.mode = true; // Pass XML object to push msg and response handlers
    
    if( this.oc.connect() )
    {
        this.joinApp( name );
        this.start();        
    }
    else
    {
        return;
    }    
  } 
  
	// Join the SimpleChat application
  public void joinApp( String name )
  {
    XmlRequest xr = oc.newRequest( this, "", "SimpleChat/Join.xma" );
    xr.setVar( "name", name );
    oc.send( xr ); 
  }

	// Main thread
  public void run()
  {
			String cmd = new String();
			boolean running = true;
			BufferedReader br = new BufferedReader( new InputStreamReader(System.in) );

	 		printHelp(); // Print the help to the console
			printPrompt();
						
			try
			{
			 	 while( running )
				 {
			 	 		 cmd = br.readLine(); 
						 
					 	 // Quit
					 	 if( cmd.toString().equals( "quit" ) )
						 {
						 		 oc.close();
								 running = false;
						 }
						 // Send message
						 else
						 {
							 	 XmlRequest xr = oc.newRequest( this, "", "SimpleChat/Send.xma" );
								 xr.setVar( "txt", "<b>" + name + ": </b>" + cmd );
            		 oc.send( xr );
								 cmd = "";
								 printPrompt();
						 }		
					}				 
			}
			catch( IOException ioe )
			{
			 		 output( "Error reading from the command line" );
			}
  }         

  // Callback for the push messages
	// Use this when the OffbeatClient.mode is true (the default)
  public void pushMsgHandler( XML data, String file, String clientID )
  {
    	//output( "Push msg" );
			
			if( file.equals( "SimpleChat/Send.xma" ) )
			{
			 		// Show the incoming push message
					output( data.firstChild.getValue() );
					printPrompt();
			}
  }
	  
  // Callback for the push messages
	// Use this when the OffbeatClient.mode is false
  public void pushMsgHandler( String data, String file, String clientID )
  {
    	//output( "Push msg" );
			
			if( file.equals( "SimpleChat/Send.xma" ) )
			{
			 		// Parse the message and show the incoming push message
					output( "Message was not parsed, check the OffbeatClient mode" );
					printPrompt();
			}
  }
  
  // Notify client disconnect
  public void clientDisconnected( String clientID, String clientName )
  {
   		output( "Client " + clientName + " disconnected" );
			printPrompt();
  }
  
  // Connection was closed
  public void closed()
  {
    	output( "Connection closed" );
  }
  
	// Connection was made with the server
	public void connected( boolean success )
	{
	 		if( success )
			{
	 		 		output( "Connected successfully" );
			}
			else
			{
			 		output( "Connection failed" );
			}
	}
	
  // Print a message to the console
  private void output( String msg )
  {
    	System.out.println( msg );
  }
	
	// Print a command prompt
	private void printPrompt()
	{
	 		System.out.print( ">" );
	}
	
	// Print short help
	private void printHelp()
	{
	 		output( "Type 'quit' to exit, type anything else to send a message.\n" );
	}
}